#include "precomp.h"

#define TILESIZE 32
#define THREADCOUNT 4
Surface ben( "assets/benoit.png" );

vec2 cmul( vec2 a, vec2 b ) { return vec2(a.x * b.x - a.y * b.y, a.x * b.y + a.y * b.x); }
float fract1( float v ) { return v - (float)(int)v; }
vec3 fract3( vec3 V ) { return vec3(fract1( V.x ), fract1( V.y ), fract1( V.z )); }
vec3 fabs3( vec3 V ) { return vec3(fabs( V.x ), fabs( V.y ), fabs( V.z )); }
vec3 mix( vec3 A, vec3 B, float s ) { return A + (B - A) * s; }
vec3 hsv( float h, float s, float v )
{
	vec4 K( 1.0f, 2.0f / 3.0f, 1.0f / 3.0f, 3.0f );
	vec3 p = fabs3( fract3( vec3(h) + K.xyz ) * 6.0f - vec3( K.w ) );
	vec3 t = p - vec3( K.x );
	return v * mix( vec3( K.x ), vec3(clamp( t.x, 0.0f, 1.0f ), clamp( t.y, 0.0f, 1.0f ), clamp( t.z, 0.0f, 1.0f )), s );
}
uint fractal( const int x, const int y, const float my, const float tm )
{
	vec2 fc( (float)x, (float)y ), j( SCRWIDTH, SCRHEIGHT );
	vec2 sp = ((fc * 2 - j) * 0.5f) * (1 / min( j.x, j.y ));
	float it = 0, m = my == 0 ? 0 : my / j.y - 0.5f, u = expf( m * 8 );
	vec2 p = sp * (u * 0.016f) - vec2(0.805f, -0.176f), z = p, c = p, dz(1, 0);
	for (float i = 0; i < 1024; i += 1)
	{
		dz = vec2( 2 * (z.x * dz.x - z.y * dz.y) + 1, 2 * (z.x * dz.y + z.y * dz.x));
		z = cmul( z, z ) + c;
		float a = sinf( tm * 1.5f + i * 2 ) * 0.3f + i * 1.3f;
		vec2 t = vec2(cosf( a ) * z.x + sinf( a ) * z.y, -sinf( a ) * z.x + cosf( a ) * z.y);
		if (fabs( t.x ) > 2 && fabs( t.y ) > 2) { it = i; break; }
	}
	if (it == 0.0f) return 0;
	float z2 = z.x * z.x + z.y * z.y, t = log( z2 ) * sqrtf( z2 ) / dz.length(), r = sqrtf( z2 );
	float q = u * 0.016f * (1 / j.x + 1 / j.y), d = j.length(), w = q * d / 400;
	float s = q * d / 80, f = 0, g = 0;
	if (t < q) f = t / q, g = 1; else f = min( s / (t + s - q) + 1 / (r + 1), 1.f ), g = min( w / (t + w - q), 1.f );
	vec3 h = hsv( it / 32 + 0.4f, 1 - g, f ) * 255;
	int a = (int)clamp( h.z, 0.f, 255.f ), b = (int)clamp( h.y, 0.f, 255.f ), e = (int)clamp( h.x, 0.f, 255.f );
	return AddBlend( (a << 16) + (b << 8) + e, ben.GetBuffer()[x + y * 512] );
}
void drawtile( Surface* screen, const int x, const int y )
{
	for( int v = 0; v < TILESIZE; v++ ) for( int u = 0; u < TILESIZE; u++ )
		screen->Plot( x + u, y + v, fractal( x + u, y + v, 280, 10.0f ) );
}

void Game::Tick( float deltaTime )
{
	// draw one line of pixels
	static int xtiles = SCRWIDTH / TILESIZE, ytiles = SCRHEIGHT / TILESIZE;
	static int tileCount = xtiles * ytiles;
	timer t;
	// #pragma omp parallel for
	for( int i = 0; i < tileCount; i++ )
	{
		int tx = i % xtiles;
		int ty = i / xtiles;
		drawtile( screen, tx * TILESIZE, ty * TILESIZE );
	}
	static float avg = 0, count = 0, duration, frame = 0;
	duration = t.elapsed(), frame += 1;
	if (frame > 6) avg += duration, count += 1;
	printf( "duration: %7.2fms (avg %7.2fms)\n", duration, avg / count );
}